import os
import platform
import sys
from pathlib import Path


def in_jupyter_notebook():
    return "JPY_PARENT_PID" in os.environ


def is_in_docker():
    return Path("/.dockerenv").exists()


def is_ubuntu():
    if platform.system() != "Linux":
        return False
    try:
        with open("/etc/os-release") as f:  # noqa: PTH123
            for line in f:
                if line.startswith("ID="):
                    return line.strip().split("=")[1].strip('"').lower() == "ubuntu"
    except FileNotFoundError:
        return False
    return False


def is_in_huggingface():
    return os.getenv("SPACE_HOST")


def is_in_vercel() -> bool:
    return os.getenv("VERCEL") == "1"


def is_in_temboio():
    return os.getenv("VISUALLY_MEASURED_DOWITCHER_TMPBOAI_PORT")


def is_in_windows() -> bool:
    return platform.system() == "Windows"


def is_in_testing() -> bool:
    """是否处于测试环境"""
    return "pytest" in sys.modules or os.getenv("PYTEST_CURRENT_TEST")


def init_env():
    sys.path.insert(0, str(Path("./mtmlib").absolute()))
    sys.path.insert(0, str(Path("./mtmtrain").absolute()))



from typing import Literal

from camelCasing import camelCasing
from pydantic import BaseModel, Field

registered_forms: dict[str, type[BaseModel]] = {}


def get_form_by_name(name: str) -> type[BaseModel]:
    return registered_forms.get(name)


def mtform(name: str | None = None):
    """
    表单装饰器，用于注册表单并设置表单属性
    """

    def decorator(cls: type[BaseModel]):
        nonlocal name
        if name is None:
            name = camelCasing.toCamelCase(cls.__name__)
        if name in registered_forms:
            msg = f"Form with name '{name}' already exists"
            raise ValueError(msg)
        registered_forms[name] = cls
        return cls

    return decorator


class FormFieldSchema(BaseModel):
    name: str
    placeholder: str | None = None
    valueType: str | None = None
    defaultValue: str | None = None
    description: str | None = None
    label: str | None = None
    type: str | None = None


class MtForm(BaseModel):
    properties: dict[str, FormFieldSchema]
    title: str
    type: str = Field(default="object")
    variant: Literal["default"] = Field(default="default")

    class Config:  # noqa: D106
        arbitrary_types_allowed = True



import logging
import subprocess

logger = logging.getLogger()


class TagVersionError(Exception):
    """自定义异常类, 用于处理版本号错误"""


def exec_shell_command(*args):
    try:
        result = (
            subprocess.check_output(args, stderr=subprocess.STDOUT).decode().strip()  # noqa: S603
        )
        return result, None  # noqa: TRY300
    except subprocess.CalledProcessError as e:
        return None, e


def read_tag():
    current_tag, err = exec_shell_command("git", "describe", "--tags")
    if err:
        return current_tag, err

    current_tag_rev, _ = exec_shell_command("git", "describe", "--tags", "--abbrev=0")
    if current_tag_rev == current_tag:
        return current_tag[1:], None

    short_commit, _ = exec_shell_command("git", "rev-parse", "--short", "HEAD")
    version = parse_version(current_tag_rev[1:])
    return f"{version}-{short_commit}", None


# def patch_git_tag_version():
#     """版本号的补丁版本加1"""
#     version = read_tag_version_next()
#     new_patch_version = f"v{version['major']}.{version['minor']}.{version['patch']}"

#     # 必须 先 commit 再 create tag。 因为tag 直接跟当前状态相关。
#     bash(f"""git add -A
#         git commit -m "{new_patch_version}" --allow-empty
#         git push --follow-tags
#     """)

#     _, tag_creation_err = exec_shell_command("git", "tag", new_patch_version)
#     if tag_creation_err:
#         msg = f"创建标签失败: {tag_creation_err}"
#         raise TagVersionError(msg)

#     return new_patch_version


def parse_version(version_str):
    parts = version_str.split(".")
    return {"major": int(parts[0]), "minor": int(parts[1]), "patch": int(parts[2])}


# def read_tag_version_next():
#     """读取下一个版本号"""
#     current_tag, err = exec_shell_command("git", "describe", "--tags")

#     if err:
#         msg = f"无法读取当前标签: {err}"
#         raise TagVersionError(msg)

#     current_tag_rev, _ = exec_shell_command("git", "describe", "--tags", "--abbrev=0")
#     version = parse_version(current_tag_rev.lstrip("v"))
#     version["patch"] += 1
#     logger.info(
#         "当前tag %s, next tag: %s, current_tag_rev: %s",
#         current_tag,
#         version,
#         current_tag_rev,
#     )
#     return version



from pathlib import Path

import httpx


# def download_file(url: str, dest: Path):
#     response = httpx.get(url, stream=True)
#     response.raise_for_status()
#     dest.parent.mkdir(parents=True, exist_ok=True)
#     with Path.open(dest, "wb") as f:
#         for chunk in response.iter_content(chunk_size=8192):
#             f.write(chunk)
#     dest.chmod(0o755)
async def download_file(url: str, dest: str | Path):
    dest = Path(dest)
    with httpx.Client(follow_redirects=True) as client:
        with client.stream("GET", url) as response:
            response.raise_for_status()
            dest.parent.mkdir(parents=True, exist_ok=True)
            with dest.open("wb") as f:
                for chunk in response.iter_bytes():
                    f.write(chunk)
        # dest.chmod(0o755)



def download_file_to_dir(url: str, dest: Path):
    """
    下载文件到指定目录, 文件名使用 url 对应的文件名
    """
    # 提取文件名并拼接到目标路径
    filename = url.split("/")[-1]
    dest_file = dest / filename

    # 创建目标文件的父目录
    dest_file.parent.mkdir(parents=True, exist_ok=True)

    try:
        with httpx.stream("GET", url) as response:
            response.raise_for_status()
            with dest_file.open("wb") as f:
                for chunk in response.iter_content(chunk_size=8192):
                    if chunk:
                        f.write(chunk)

        # 设置文件权限
        dest_file.chmod(0o755)

    except httpx.HTTPStatusError as e:
        print(f"HTTP error occurred: {e}")
    except Exception as e:
        print(f"An error occurred: {e}")
